#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 Javier Escalada Gómez
All rights reserved.
License: BSD 3-Clause Clear License (see LICENSE for details)
"""

# WARNING: This is a copy & paste & patch version from
# https://github.com/python/cpython/blob/3.10/Lib/json/tool.py
# For future versions follow the same algorithm.

from custom_json_encoder import CustomJSONEncoder

import argparse
import json
import sys
from pathlib import Path

#region patch
def config_indent_hint(options):
    def indent_hint(path, collection, indent, width):
        if options.indent is not None:
            return True
        if len(collection) == 0:
            return False
        if len(path) == 0:
            return True
        if path[-1] in options.indent_after:
            return True
        return False
    return indent_hint
#endregion

def main():
#region patch
    #prog = 'python -m json.tool'
    prog = 'custom_json_encoder'
#endregion
    description = ('A simple command line interface for json module '
                   'to validate and pretty-print JSON objects.')
    parser = argparse.ArgumentParser(prog=prog, description=description)
    parser.add_argument('infile', nargs='?',
                        type=argparse.FileType(encoding="utf-8"),
                        help='a JSON file to be validated or pretty-printed',
                        default=sys.stdin)
    parser.add_argument('outfile', nargs='?',
                        type=Path,
                        help='write the output of infile to outfile',
                        default=None)
    parser.add_argument('--sort-keys', action='store_true', default=False,
                        help='sort the output of dictionaries alphabetically by key')
    parser.add_argument('--no-ensure-ascii', dest='ensure_ascii', action='store_false',
                        help='disable escaping of non-ASCII characters')
    parser.add_argument('--json-lines', action='store_true', default=False,
                        help='parse input using the JSON Lines format. '
                        'Use with --no-indent or --compact to produce valid JSON Lines output.')
    group = parser.add_mutually_exclusive_group()
    group.add_argument('--indent',
#region patch
                       default=None,
#endregion
                       type=int,
                       help='separate items with newlines and use this number '
                       'of spaces for indentation')
#region patch
    group.add_argument('--indent-after', action='append', default=[],
                       metavar='KEY',
                       help='indent after the given key using '
                       '--indent-after-indentation spaces')
#endregion
    group.add_argument('--tab', action='store_const', dest='indent',
                       const='\t', help='separate items with newlines and use '
                       'tabs for indentation')
#region patch
    # group.add_argument('--no-indent', action='store_const', dest='indent',
    #                    const=None,
    #                    help='separate items with spaces rather than newlines')
#endregion
    group.add_argument('--compact', action='store_true',
                       help='suppress all whitespace separation (most compact)')
#region patch
    parser.add_argument('--indent-after-width', default=80, type=int, metavar='AMOUNT',
                       help='set the width of the output line when '
                        '--indent-after is active')
    parser.add_argument('--indent-after-indentation', default=2, type=int,
                        metavar='AMOUNT',
                        help='use this number of spaces for indentation when '
                        '--indent-after is active')
#endregion
    options = parser.parse_args()

    dump_args = {
        'sort_keys': options.sort_keys,
        'indent': options.indent or options.indent_after_indentation,
        'ensure_ascii': options.ensure_ascii,
#region patch
        'cls': CustomJSONEncoder,
        'width': options.indent_after_width,
        'indent_hint': config_indent_hint(options),
#endregion
    }
    if options.compact:
        dump_args['indent'] = None
        dump_args['separators'] = ',', ':'

    with options.infile as infile:
        try:
            if options.json_lines:
                objs = (json.loads(line) for line in infile)
            else:
                objs = (json.load(infile),)

            if options.outfile is None:
                out = sys.stdout
            else:
                out = options.outfile.open('w', encoding='utf-8')
            with out as outfile:
                for obj in objs:
                    json.dump(obj, outfile, **dump_args)
                    outfile.write('\n')
        except ValueError as e:
            raise SystemExit(e)


if __name__ == '__main__':
    try:
        main()
    except BrokenPipeError as exc:
        sys.exit(exc.errno)



# -*- coding: utf-8 -*-
"""
A JSON encoder that allows customizing the indentation based on the content and
the width of the line.

Copyright (c) 2022 Javier Escalada Gómez
All rights reserved.
License: BSD 3-Clause Clear License (see LICENSE for details)
"""

__author__ = "Javier Escalada Gómez"
__email__ = "kerrigan29a@gmail.com"
__version__ = "0.2.1"
__license__ = "BSD 3-Clause Clear License"

# Inspiration: https://gist.github.com/jannismain/e96666ca4f059c3e5bc28abb711b5c92

from json import JSONEncoder, dump, dumps


def indentation_policy_default(path, collection, indent, width):
    """Default indentation policy.

    Indent always except when the collection is empty.

    Args:
        path (list): The path of the current collection.
        collection: The collection to be indented.
        indent: The current indentation.
        width (int): The width of the line.
    """
    if len(collection) == 0:
        return False
    return True


class CustomJSONEncoder(JSONEncoder):
    """Custom JSON encoder.

    This class allows customizing the indentation based on the content and the
    width of the line.
    """

    def __init__(self, *args, **kwargs):
        if kwargs.get("indent") is None:
            kwargs.update({"indent": 2})
        if kwargs.get("separators") is None:
            kwargs.update({"separators": (",", ": ")})
        self.compact_item_separator, self.compact_key_separator = kwargs.pop(
            "compact_separators", (", ", ": "))
        self.indentation_policy = kwargs.pop("indentation_policy",
            indentation_policy_default)
        self.width = kwargs.pop("width", 0)
        super().__init__(*args, **kwargs)

    def iterencode(self, o, *args, **kwargs):
        return self._wrap(self._iterencode(o, []))

    def encode(self, o):
        return "".join(self.iterencode(o))

    def _parent_encode(self, o):
        return JSONEncoder.iterencode(self, o)

    def _iterencode(self, o, path):
        depth = len(path)

        if isinstance(o, (list, tuple)):
            nl_sep, ind_sep, it_sep, _ = self._config(path, o)

            yield "["
            for i, el in enumerate(o):
                if i > 0:
                    yield it_sep
                yield from (nl_sep, (depth + 1) * ind_sep)
                yield from self._iterencode(el, path + [i])
            yield from (nl_sep, depth * ind_sep, "]")

        elif isinstance(o, dict):
            nl_sep, ind_sep, it_sep, key_sep = self._config(path, o)

            yield "{"
            for i, (k, v) in enumerate(o.items()):
                if not isinstance(k, (str, int, float, bool)) and k is not None:
                    raise TypeError(
                        f"keys must be str, int, float, bool or None, not {k.__class__.__name__}"
                    )
                if i > 0:
                    yield it_sep
                yield from (nl_sep, (depth + 1) * ind_sep)
                yield from self._parent_encode(k)
                yield self.key_separator
                yield from self._iterencode(v, path + [k])
            yield from (nl_sep, depth * ind_sep, "}")

        else:
            yield from self._parent_encode(o)
    
    def _indent_str(self):
        if isinstance(self.indent, int):
            return self.indent * " "
        return self.indent

    def _indent_len(self):
        if isinstance(self.indent, int):
            return self.indent
        return len(self.indent)

    def _config(self, path, collection):
        if self.indentation_policy(path, collection, self.indent, self.width):
            return "\n", self._indent_str(), self.item_separator, self.key_separator
        return "", "", self.compact_item_separator, self.compact_key_separator

    def _wrap(self, tokens):
        col = 1
        prefix = ""
        for token in tokens:
            if token == "":
                continue

            if not self.width:
                yield token
                continue

            if token == "\n":
                col = 1
                yield token
                continue

            if col == 1 and all(c in [" ", "\t"] for c in token):
                prefix = token
                col += len(token)
                yield token
                continue

            col += len(token)
            if col > self.width + 1:
                if token in [
                    self.compact_item_separator,
                    self.compact_key_separator,
                    self.item_separator,
                    self.key_separator,
                    "[",
                    "]",
                    "{",
                    "}",
                ]:
                    yield token
                    continue

                new_col = 1 + len(prefix) + self._indent_len() + len(token)
                if new_col <= self.width + 1:
                    yield from ("\n", prefix, self._indent_str())
                    col = new_col

            yield token


def _normalize_kwargs(kwargs, indentation_policy, width):
    if kwargs.get("indent", None) is None:
        kwargs.pop("indentation_policy", None)
        kwargs.pop("width", None)
    else:
        kwargs["cls"] = CustomJSONEncoder
        kwargs["indentation_policy"] = kwargs.get("indentation_policy",
            indentation_policy or indentation_policy_default)
        kwargs["width"] = kwargs.get("width", width)
    return kwargs


def wrap_dump(indentation_policy=None, width=0):
    """Wrap the json.dump function to use the CustomJSONEncoder.

    If indent is None, no indentation is used, otherwise the indentation is
    determined by the indentation_policy and the width.
    """
    def wrapper(*args, **kwargs):
        return dump(*args, **_normalize_kwargs(kwargs, indentation_policy,
            width))
    return wrapper


def wrap_dumps(indentation_policy=None, width=0):
    """Wrap the json.dumps function to use the CustomJSONEncoder.

    If indent is None, no indentation is used, otherwise the indentation is
    determined by the indentation_policy and the width.
    """
    def wrapper(*args, **kwargs):
        return dumps(*args, **_normalize_kwargs(kwargs, indentation_policy,
            width))
    return wrapper



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Copyright (c) 2022 Javier Escalada Gómez
All rights reserved.
License: BSD 3-Clause Clear License (see LICENSE for details)
"""

from setuptools import setup, find_packages
from pathlib import Path
from custom_json_encoder import __version__, __author__, __email__, __license__, __doc__

base = Path(__file__).parent
long_description = (base / "README.md").read_text()

setup(
    name="custom_json_encoder",
    version=__version__,
    url="https://github.com/Kerrigan29a/py_custom_json_encoder",
    project_urls={
        "Source": "https://github.com/Kerrigan29a/py_custom_json_encoder",
    },
    author=__author__,
    author_email=__email__,
    description=__doc__,
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=[],
    license=__license__,
    entry_points={
        "console_scripts": [
            "custom_json_encoder=custom_json_encoder.__main__:main",
        ],
    },
    # From https://pypi.org/classifiers/
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Utilities",
    ],
    platforms="any",
    keywords=["json", "encoder", "pretty-print", "width"],
)
